<?php

namespace Drupal\flysystem\Adapter;

use League\Flysystem\Config;
use League\Flysystem\FileAttributes;
use League\Flysystem\FilesystemAdapter;
use League\Flysystem\InvalidVisibilityProvided;
use League\Flysystem\UnableToCopyFile;
use League\Flysystem\UnableToCreateDirectory;
use League\Flysystem\UnableToDeleteDirectory;
use League\Flysystem\UnableToDeleteFile;
use League\Flysystem\UnableToMoveFile;
use League\Flysystem\UnableToReadFile;
use League\Flysystem\UnableToWriteFile;

/**
 * The Flysystem MissingAdapter definition for integration with Drupal.
 */
class MissingAdapter implements FilesystemAdapter {

  /**
   * {@inheritdoc}
   */
  public function fileExists(string $path): bool {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function directoryExists(string $path): bool {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function write(string $path, string $contents, Config $config): void {
    throw new UnableToWriteFile();
  }

  /**
   * {@inheritdoc}
   */
  public function writeStream(string $path, $contents, Config $config): void {
    throw new UnableToWriteFile();
  }

  /**
   * {@inheritdoc}
   */
  public function read(string $path): string {
    return '';
  }

  /**
   * {@inheritdoc}
   */
  public function readStream(string $path) {
    throw new UnableToReadFile();
  }

  /**
   * {@inheritdoc}
   */
  public function delete(string $path): void {
    throw new UnableToDeleteFile();
  }

  /**
   * {@inheritdoc}
   */
  public function deleteDirectory(string $dirname): void {
    throw new UnableToDeleteDirectory();
  }

  /**
   * {@inheritdoc}
   */
  public function createDirectory(string $dirname, Config $config): void {
    throw new UnableToCreateDirectory();
  }

  /**
   * {@inheritdoc}
   */
  public function setVisibility(string $path, string $visibility): void {
    throw new InvalidVisibilityProvided();
  }

  /**
   * {@inheritdoc}
   */
  public function visibility(string $path): FileAttributes {
    return new FileAttributes('');
  }

  /**
   * {@inheritdoc}
   */
  public function mimeType(string $path): FileAttributes {
    return new FileAttributes('');
  }

  /**
   * {@inheritdoc}
   */
  public function lastModified(string $path): FileAttributes {
    return new FileAttributes('');
  }

  /**
   * {@inheritdoc}
   */
  public function fileSize(string $path): FileAttributes {
    return new FileAttributes('');
  }

  /**
   * {@inheritdoc}
   */
  public function listContents(string $directory = '', $deep = FALSE): iterable {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function move(string $source, string $destination, Config $config): void {
    throw new UnableToMoveFile();
  }

  /**
   * {@inheritdoc}
   */
  public function copy(string $source, string $destination, Config $config): void {
    throw new UnableToCopyFile();
  }

}
