<?php

namespace Drupal\flysystem;

use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\Core\DependencyInjection\ServiceProviderBase;
use Drupal\Core\Site\Settings;
use Drupal\flysystem\Asset\SchemeExtensionTrait;

/**
 * Flysystem dependency injection container.
 *
 * @todo We will need to keep this, but extend ServiceProviderBase so we can alter core service providers
 * @see https://www.bounteous.com/insights/2017/04/19/drupal-how-override-core-drupal-8-service
 * @see https://www.drupal.org/docs/drupal-apis/services-and-dependency-injection/altering-existing-services-providing-dynamic-services
 */
class FlysystemServiceProvider extends ServiceProviderBase {

  use SchemeExtensionTrait;

  /**
   * {@inheritdoc}
   */
  public function register(ContainerBuilder $container) {

    // Do we really need to do this?
    // Commenting out: $this->swapDumper($container, 'js')
    // Commentint out: $this->swapDumper($container, 'css')
    foreach (Settings::get('flysystem', []) as $scheme => $settings) {

      // Just some sanity checking, so things don't explode.
      if (empty($settings['driver'])) {
        continue;
      }
      // Registers the FlysystemStreamWrapper.
      // @todo Revisit if this is needed,
      $container
        ->register('flysystem_stream_wrapper.' . $scheme, 'Drupal\flysystem\FlysystemStreamWrapper')
        ->addTag('stream_wrapper', ['scheme' => $scheme]);

      // Register the path processors for local files.
      // @todo review the path processor to determine if this is needed.
      if ($settings['driver'] === 'local' && !empty($settings['config']['public'])) {
        $container
          ->register('flysystem.' . $scheme . '.path_processor', 'Drupal\flysystem\PathProcessor\LocalPathProcessor')
          ->addTag('path_processor_inbound', ['priority' => 400])
          ->addArgument($scheme);
      }
    }
  }

  /**
   * {@inheritdoc}
   *
   * @todo Determine if this is needed if not remove.
   */
  /*
  public function alter(ContainerBuilder $container) {
  // Replace Drupal\Core\File\FileSystem service with
  // Drupal\flysystem\FileSystem.
  if ($container->hasDefinition('file_system')) {
  $definition = $container->getDefinition('file_system');
  $definition->setClass('Drupal\flysystem\FileSystem');
  // Uncomment and implement if additional services are needed here.
  // ->addArgument(new Reference('entity_type.manager'));.
  }
  }
   */
  /**
   * Swaps the js/css dumper if a scheme is configured to serve it.
   *
   * @param \Drupal\Core\DependencyInjection\ContainerBuilder $container
   *   The container.
   * @param string $extension
   *   The file extension to swap.
   *
   * @todo Determine if this is needed, if not remove.
   */
  /*
  protected function swapDumper(ContainerBuilder $container, $extension) {
  if (!$container->has('asset.' . $extension . '.dumper')) {
  return;
  }

  if (!$container->has('asset.' . $extension . '.collection_optimizer')) {
  return;
  }

  // Don't rewrite if there's nothing to change.
  if ($this->getSchemeForExtension($extension) === 'public') {
  return;
  }

  $container
  ->getDefinition('asset.' . $extension . '.dumper')
  ->setClass('Drupal\flysystem\Asset\AssetDumper');
  $container
  ->getDefinition('asset.' . $extension . '.collection_optimizer')
  ->setClass('Drupal\flysystem\Asset\\' . ucfirst($extension) . 'CollectionOptimizer');

  if ($extension === 'css') {
  $container
  ->getDefinition('asset.css.optimizer')
  ->setClass('Drupal\flysystem\Asset\CssOptimizer');
  }
  }
   */

}
